import re
from base64 import b64decode

from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5


class SecurityManager:
    """
    Class responsible for the verification of the incoming signature using the
    public key provided from Freshchat
    """

    def __init__(self, public_key: str) -> None:
        self.public_key = public_key

    def key_parse(self) -> str:
        x = re.match(
            "^-----BEGIN RSA PUBLIC KEY-----((.+)|\n(.+)\n)-----END RSA PUBLIC KEY-----\n?$",
            self.public_key,
        )
        if x:
            return x.group(1).strip()
        return None

    @property
    def rsa_key(self) -> RSA:
        """
        Property returns RSA instance from the public key
        """
        data = b64decode(self.key_parse())
        return RSA.importKey(data)

    def verify_signature(self, signature: str, data: str) -> bool:
        """
        Method which verifies if the data are signed correctly
        :param signature: sha256withrsa signature
        :param data: signed data
        :return: the result of the verification
        """
        rsa_key = self.rsa_key
        digest = SHA256.new(data)
        signer = PKCS1_v1_5.new(rsa_key)
        return signer.verify(digest, b64decode(signature))



from typing import Any, AnyStr, Dict, Optional

import aiohttp
from cafeteria.logging import LoggedObject

from freshchat.client.configuration import FreshChatConfiguration
from freshchat.client.exceptions import HttpResponseCodeError
from freshchat.client.responses import FreshChatResponse


class FreshChatClient(LoggedObject):
    """
    Class represents an HTTP client
    """

    def __init__(self, config: FreshChatConfiguration) -> None:
        self.config = config

    async def request(
        self,
        method: str,
        endpoint: str,
        params: Optional[Dict[AnyStr, Any]] = None,
        json: Optional[Dict[AnyStr, Any]] = None,
        headers: Optional[Dict[AnyStr, Any]] = None,
    ) -> FreshChatResponse:
        """

        :param method: http request method
        :param endpoint: Resource endpoint
        :param params: request parameters
        :param json: request json body
        :param headers: Additional request headers
        """
        request_headers = (
            headers.update(self.config.authorization_header)
            if headers
            else self.config.authorization_header
        )

        url = self.config.get_url(endpoint=endpoint)

        self.logger.debug(
            "%s %s \n> params: %s\n> headers: %s%s",
            method,
            url,
            params,
            headers,
            f"\n> body: {json}" if json else "",
        )
        async with aiohttp.ClientSession() as session:
            async with session.request(
                method=method,
                url=url,
                params=params,
                json=json,
                headers=request_headers,
            ) as response:
                response = await FreshChatResponse.load(response=response)
                self.logger.debug(
                    "%s %s %d \n< %s", method, url, response.http.status, response.body
                )

                if 200 <= response.http.status < 300:
                    return response
                raise HttpResponseCodeError(response)

    async def get(
        self,
        endpoint: str,
        params: Optional[Dict[AnyStr, Any]] = None,
        headers: Optional[Dict[str, Any]] = None,
    ) -> FreshChatResponse:
        """
        Method used for the get requests

        :param endpoint: Resource endpoint
        :param params: request parameters
        :param headers: Additional request headers
        """
        return await self.request(
            method="GET", endpoint=endpoint, params=params, headers=headers
        )

    async def post(
        self,
        endpoint: str,
        params: Optional[Dict[AnyStr, Any]] = None,
        json: Optional[Dict[AnyStr, AnyStr]] = None,
        headers: Optional[Dict[str, Any]] = None,
    ) -> FreshChatResponse:
        """
        Method used for the post requests

        :param endpoint: Resource endpoint
        :param params: request parameters
        :param json: request json body
        :param headers: Additional request headers
        """
        return await self.request(
            method="POST", endpoint=endpoint, params=params, json=json, headers=headers
        )

    async def put(
        self,
        endpoint: str,
        params: Optional[Dict[AnyStr, Any]] = None,
        json: Optional[Dict[AnyStr, AnyStr]] = None,
        headers: Optional[Dict[str, Any]] = None,
    ) -> FreshChatResponse:
        """
        Method used for the put requests

        :param endpoint: Resource endpoint
        :param params: request parameters
        :param json: request json body
        :param headers: Additional request headers
        """
        return await self.request(
            method="PUT", endpoint=endpoint, params=params, json=json, headers=headers
        )

    def __repr__(self):
        return (
            f"{self.__class__.__name__}<{hex(id(self))}> (config={repr(self.config)})"
        )



from dataclasses import asdict, dataclass, field
from typing import Any, AnyStr, ClassVar, Dict, List, Optional, Union

from freshchat.client.client import FreshChatClient


@dataclass
class User:
    """
    Class which represents a freshchat user. A user can be an external user or an
    agent
    """

    id: Optional[str] = field(default=None)
    created_time: Optional[str] = field(default=None)
    email: Optional[str] = field(default=None)
    first_name: Optional[str] = field(default=None)
    last_name: Optional[str] = field(default=None)
    phone: Optional[str] = field(default=None)
    avatar: Optional[Dict[AnyStr, AnyStr]] = field(default_factory=dict)
    social_profiles: Optional[List[Dict[AnyStr, AnyStr]]] = field(default_factory=list)
    properties: Optional[List[Dict[AnyStr, AnyStr]]] = field(default_factory=list)
    endpoint: ClassVar[str] = "/users"

    @property
    def get_endpoint(self) -> str:
        return f"{self.endpoint}/{self.id}"

    @classmethod
    async def create(cls, client: FreshChatClient, **kwargs) -> "User":
        """
        Creates a new user instance with the given kwargs
        """
        user = cls(**kwargs)
        response = await client.post(endpoint=user.endpoint, json=asdict(user))
        return cls(**response.body)

    @classmethod
    async def get(cls, client: FreshChatClient, user_id: str) -> "User":
        """
        Returns an existing user based on the given user_id
        """
        user = cls(id=user_id)
        response = await client.get(user.get_endpoint)
        return cls(**response.body)


@dataclass
class Message:
    """
    Class which represents freshchat message format
    """

    created_time: Optional[str] = field(default=None)
    id: Optional[str] = field(default=None)
    app_id: str = field(default=None)
    actor_type: Optional[str] = "user"
    actor_id: str = field(default=None)
    channel_id: Optional[str] = field(default=None)
    conversation_id: Optional[str] = field(default=None)
    message_type: str = "normal"
    message_parts: List[Dict[AnyStr, AnyStr]] = field(default_factory=list)

    @property
    def endpoint(self) -> str:
        return f"/conversations/{self.conversation_id}/messages"


@dataclass
class Conversation:
    """
    Class which represents freshchat conversation format
    """

    conversation_id: Optional[str] = field(default=None)
    app_id: str = field(default=None)
    channel_id: str = field(default=None)
    status: Optional[str] = field(default="new")
    agents: Optional[List[User]] = field(default_factory=list)
    users: List[User] = field(default_factory=list)
    messages: List[Message] = field(default_factory=list)
    endpoint: ClassVar[str] = "/conversations"

    @property
    def get_endpoint(self):
        """
        Property returns endpoint for the GET method
        :return:
        """
        return f"{self.endpoint}/{self.conversation_id}"

    @property
    def user_id(self):
        return self.users[0].id

    @classmethod
    async def create(
        cls,
        client: FreshChatClient,
        user_id: str,
        channel_id: Optional[str] = None,
        init_message: Optional[str] = None,
    ) -> "Conversation":
        """
        Create a new conversation instance
        :param client: FreshChatClient to make the necessary requests
        :param user_id: the id of the user who creates the conversation
        :param channel_id: the id of the channel which the conversation will be assigned
        :param init_message: the initial message of the conversation
        :return: an instance of the class with the additional information returned from
        Freshchat API
        """
        user = await User().get(client=client, user_id=user_id)
        conversation_body = {
            "app_id": client.config.app_id,
            "channel_id": channel_id or client.config.default_channel_id,
            "users": [asdict(user)],
            "messages": [
                asdict(
                    Message(
                        **{
                            "app_id": client.config.app_id,
                            "actor_id": user.id,
                            "channel_id": channel_id
                            or client.config.default_channel_id,
                            "message_parts": [{"text": {"content": init_message}}],
                        }
                    )
                )
            ],
        }

        conversation = cls(**conversation_body)

        response = await client.post(
            endpoint=conversation.endpoint, json=asdict(conversation)
        )
        conversation = cls(**response.body)
        conversation.users = [user]
        return conversation

    @classmethod
    async def get(
        cls, client: FreshChatClient, conversation_id: str, user_id: str
    ) -> "Conversation":
        """
        Method which returns an existing conversation based on the conversation_id
        :param client: FreshChatClient to make the necessary requests
        :param conversation_id: the id of the conversation
        :param user_id: the id of the user
        :return: an instance of the class with the additional information returned from
        Freshchat API
        """

        user = await User().get(client=client, user_id=user_id)
        conversation = cls(conversation_id=conversation_id)
        response = await client.get(conversation.get_endpoint)
        conversation = cls(**response.body)
        conversation.users = [user]
        return conversation

    async def send(
        self,
        client: FreshChatClient,
        message: str,
        **kwargs: Union[str, List[Dict[str, str]]],
    ) -> Message:
        """
        Sends a message to an existing conversation

        :param client: FreshChatClient to make the necessary requests
        :param message: message to be send in the  conversation
        :param kwargs: Additional message model properties to configure
        :return: am instance of the Message class with the additional information
        returned from Freshchat API
        """
        message_parts = [
            {"text": {"content": message}},
            *kwargs.pop("message_parts", []),
        ]
        properties = {
            "conversation_id": self.conversation_id,
            "actor_id": self.user_id,
            "message_parts": message_parts,
        }
        properties.update(kwargs)

        message_model = Message(**properties)
        response = await client.post(
            endpoint=message_model.endpoint, json=asdict(message_model)
        )
        return Message(**response.body)

    async def resolve(self, client: FreshChatClient) -> "Conversation":
        """
        Method which resolves the existing Conversation
        :param client: FreshChatClient to make the necessary requests
        :return:  an instance of the class with the additional information returned from
        Freshchat API
        """
        status = {"status": "resolved"}
        response = await client.put(endpoint=self.get_endpoint, json=status)
        return Conversation(**response.body)


@dataclass
class Group:
    """
    Class which represents freshchat group format
    """

    id: Optional[str] = field(default=None)
    name: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None)
    routing_type: Optional[str] = field(default=None)


@dataclass
class Channel:
    """
    Class which represents freshchat channel object
    """

    id: Optional[str] = field(default=None)
    icon: Optional[Dict[AnyStr, AnyStr]] = field(default_factory=dict)
    updated_time: Optional[str] = field(default=None)
    enabled: Optional[bool] = field(default=None)
    public: Optional[bool] = field(default=None)
    name: Optional[str] = field(default=None)
    tags: Optional[List[Any]] = field(default_factory=list)
    welcome_message: Optional[Dict[Any, Any]] = field(default_factory=dict)


@dataclass
class Channels:
    """
    Class which represents freshchat channels. It is the returning
    value of get channels request
    """

    channels: List[Channel] = field(default_factory=list)
    pagination: Optional[Dict[AnyStr, AnyStr]] = field(default_factory=dict)
    links: Optional[Dict[AnyStr, AnyStr]] = field(default_factory=dict)
    last_page: Optional[Dict[AnyStr, AnyStr]] = field(default_factory=dict)
    endpoint: ClassVar[str] = "/channels"

    @classmethod
    async def get(cls, client: FreshChatClient) -> List:
        """
        Returns a list of Channel
        """
        response = await client.get(Channels().endpoint)
        return Channels(**response.body).channels


@dataclass
class Actor:
    actor_type: Optional[str] = field(default=None)
    actor_id: Optional[str] = field(default=None)



import os
from dataclasses import dataclass, field
from typing import AnyStr, Dict, Optional
from urllib.parse import urljoin


@dataclass
class FreshChatConfiguration:
    """
    Class represents the base configuration for Freshchat
    """

    app_id: str
    token: str = field(repr=False)
    default_channel_id: Optional[str] = field(default=None)
    default_initial_message: Optional[str] = field(default=None)
    url: Optional[str] = field(
        default_factory=lambda: os.environ.get(
            "FRESHCHAT_API_URL", "https://api.freshchat.com/v2/"
        )
    )

    @property
    def authorization_header(self) -> Dict[AnyStr, AnyStr]:
        """
        Property which returns the proper format of the authorization header
        """

        return {
            "Authorization": f"Bearer {self.token}"
            if "Bearer" not in self.token
            else self.token
        }

    def get_url(self, endpoint: str) -> str:
        """
        Method responsible to build the url using the given endpoint

        :param endpoint: String with the endpoint which needs to attached to URL
        :return: a string which represents URL
        """
        return urljoin(self.url, endpoint.lstrip("/"))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['freshchat', 'freshchat.client', 'freshchat.models', 'freshchat.webhook']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.6,<4.0', 'cafeteria>=0.19.0', 'pycrypto>=2.6,<3.0']

setup_kwargs = {
    'name': 'freshchat',
    'version': '0.6.0',
    'description': 'A library provide a http client for Freshchat API',
    'long_description': '[![image](https://img.shields.io/pypi/v/freshchat.svg)](https://pypi.org/project/freshchat/)\n[![image](https://img.shields.io/pypi/l/freshchat.svg)](https://pypi.org/project/freshchat/)\n[![image](https://img.shields.io/pypi/pyversions/freshchat.svg)](https://pypi.org/project/freshchat/)\n[![image](https://readthedocs.org/projects/freshchat/badge/?version=latest&style=flat)](https://freshchat.readthedocs.io/)\n[![image](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n[![](https://github.com/twyla-ai/python-freshchat/workflows/Main%20Workflow/badge.svg)](https://github.com/twyla-ai/python-freshchat/actions)\n[![Sonarcloud Status](https://sonarcloud.io/api/project_badges/measure?project=twyla-ai_python-freshchat&metric=alert_status)](https://sonarcloud.io/dashboard?id=twyla-ai_python-freshchat)\n\n# Python Freshchat Client Library\nA python client library for [Freshchat (Live Chat Software)](https://www.freshworks.com/live-chat-software/). This library allows users to interact with the Freshchat API to perform actions on the following resources.\n\n* Users\n* Channels\n* Conversations\n\nThis library can also be used to build B2B or C2B live chat clients integrating with the Freshchat API.\n\n## Installation\n`pip install freshchat`\n\n## Documentation\nThe project documentation is available [here](https://freshchat.readthedocs.io/en/latest/). Be sure to check out the [introduction](https://freshchat.readthedocs.io/en/latest/intro.html) for usage examples.\n\n## Reporting Issues and Contributing\nThis project is maintained on [GitHub](https://github.com/twyla-ai/python-freshchat).\n',
    'author': 'Twyla Engineering',
    'author_email': 'software@twyla.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/twyla-ai/python-freshchat',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from http import HTTPStatus
from typing import Dict, Type

from freshchat.client.responses import FreshChatResponse


class FreshChatClientException(Exception):
    """
    Class to customize exceptions of the library with attribute DEFAULT_MESSAGE
    to be returned if message isn't provided
    """

    DEFAULT_MESSAGE = "Encountered a live agent client error"

    def __init__(
        self,
        response: FreshChatResponse,
        *args,
    ) -> None:
        message: str = self.DEFAULT_MESSAGE
        if isinstance(response.body, dict):
            message = response.body.get("message")
        elif isinstance(response.body, str):
            message = response.body

        super().__init__(message, *args)
        self._message = message
        self._response = response

    @property
    def message(self) -> str:
        return self._message

    @property
    def response(self) -> FreshChatResponse:
        return self._response


class FreshChatUnauthorised(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Unauthorized exception and
    sets DEFAULT_MESSAGE to `Access to the requested resource is unauthorised`
    """

    DEFAULT_MESSAGE = "Access to the requested resource is unauthorised"


class FreshChatForbidden(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Forbidden access exception
    and sets DEFAULT_MESSAGE to `Access to the requested resource  is not permitted`
    """

    DEFAULT_MESSAGE = "Access to the requested resource  is not permitted"


class FreshChatNotAllowed(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Not Allowed exception
    and sets DEFAULT_MESSAGE to `Access to the requested resource  is not allowed`
    """

    DEFAULT_MESSAGE = "Access to the requested resource  is not allowed"


class FreshChatBadRequest(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Bad Request exception
    and sets DEFAULT_MESSAGE to `Bad request, probably due to invalid syntax or
    missing required " "fields`
    """

    DEFAULT_MESSAGE = (
        "Bad request, probably due to invalid syntax or missing required " "fields "
    )


class TooManyRequests(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Too Many Requests exception
    and sets DEFAULT_MESSAGE to `Too many requests. Requests overcame the
    proper time limit`
    """

    DEFAULT_MESSAGE = "Too many requests. Requests overcame the proper time limit"


class ServerUnavailable(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Unavailable Server exception
    and sets DEFAULT_MESSAGE to `The server is currently unable to handle the request
    due to a temporary overload or scheduled maintenance"`
    """

    DEFAULT_MESSAGE = (
        "The server is currently unable to handle the request due to a "
        "temporary overload or scheduled maintenance"
    )


class ResourceNotFound(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Resource Not Found exception
    and sets DEFAULT_MESSAGE to `The requested resource was not found`
    """

    DEFAULT_MESSAGE = "The requested resource was not found"


class ServerSideError(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Server Side Error exception
    and sets DEFAULT_MESSAGE to `Something went wrong on the server side`
    """

    DEFAULT_MESSAGE = "Something went wrong on the server side"


class Conflict(FreshChatClientException):
    """
    Class inherits from FreshChatClientException represents Conflict exception
    and sets DEFAULT_MESSAGE to `The request causes data inconsistencies`
    """

    DEFAULT_MESSAGE = "The request causes data inconsistencies"


RESPONSE_CODE_TO_ERROR_MAPPING: Dict[int, Type[FreshChatClientException]] = {
    HTTPStatus.BAD_REQUEST: FreshChatBadRequest,
    HTTPStatus.NOT_FOUND: ResourceNotFound,
    HTTPStatus.INTERNAL_SERVER_ERROR: ServerSideError,
    HTTPStatus.UNAUTHORIZED: FreshChatUnauthorised,
    HTTPStatus.FORBIDDEN: FreshChatForbidden,
    HTTPStatus.METHOD_NOT_ALLOWED: FreshChatNotAllowed,
    HTTPStatus.TOO_MANY_REQUESTS: TooManyRequests,
    HTTPStatus.SERVICE_UNAVAILABLE: ServerUnavailable,
    HTTPStatus.CONFLICT: Conflict,
}


class HttpResponseCodeError:
    """
    Class responsible to return the proper exception based on the response status code
    """

    def __new__(cls, response: FreshChatResponse) -> FreshChatClientException:
        return RESPONSE_CODE_TO_ERROR_MAPPING.get(
            response.status, FreshChatClientException
        )(response=response)



from dataclasses import asdict, dataclass, field
from typing import Any, AnyStr, Dict, List, Optional

from freshchat.models import Actor, Conversation


@dataclass
class Message:
    """
    Class which represents freshchat new message event
    """

    created_time: str = field(default=None)
    id: str = field(default=None)
    actor_type: str = field(default=None)
    actor_id: str = field(default=None)
    conversation: Conversation = field(default_factory=Conversation)
    message_type: str = field(default=None)
    message_parts: List[Dict[AnyStr, AnyStr]] = field(default_factory=list)
    user_id: Optional[str] = field(default=None)

    def __post_init__(self):
        if isinstance(self.conversation, dict):
            self.conversation = Conversation(**self.conversation)

    @classmethod
    def create(cls, incoming_message: Dict[AnyStr, AnyStr]) -> "Message":
        conversation = Conversation(
            conversation_id=incoming_message.pop("conversation_id"),
            channel_id=incoming_message.pop("channel_id"),
            app_id=incoming_message.pop("app_id"),
        )
        incoming_message["conversation"] = asdict(conversation)
        return cls(**incoming_message)


@dataclass
class Reopen:
    """
    Class which represents freshchat conversation reopen event
    """

    reopener: str = field(default=None)
    reopener_id: str = field(default=None)
    conversation: Conversation = field(default_factory=Conversation)

    def __post_init__(self):
        if isinstance(self.conversation, dict):
            self.conversation = Conversation(**self.conversation)


@dataclass
class Resolve:
    """
    Class which represents freshchat conversation resolve event
    """

    resolver: str = field(default=None)
    resolver_id: str = field(default=None)
    conversation: Conversation = field(default_factory=Conversation)

    def __post_init__(self):
        if isinstance(self.conversation, dict):
            self.conversation = Conversation(**self.conversation)


@dataclass
class IncomingEvent:
    """
    Class which accepts an incoming event and based on the type creates the
    corresponding event
    """

    actor: Actor = field(default_factory=Actor)
    action: str = field(default=None)
    action_time: str = field(default=None)
    data: Any = field(default_factory=None)

    def __post_init__(self):
        if isinstance(self.data, dict):
            if "message" in self.data:
                self.data = Message.create(self.data.get("message"))
            elif "resolve" in self.data:
                self.data = Resolve(**self.data.get("resolve"))
            elif "reopen" in self.data:
                self.data = Reopen(**self.data.get("reopen"))
        if isinstance(self.actor, dict):
            self.actor = Actor(**self.actor)



import json
from typing import Any, AnyStr, Dict, Union

from aiohttp import ClientResponse

FreshChatResponseBody = Union[str, Dict[AnyStr, Any]]


class FreshChatResponse:
    """
    Class represent http response
    """

    def __init__(
        self, response: ClientResponse, body: FreshChatResponseBody = None
    ) -> None:
        self._response = response
        self._body: FreshChatResponseBody = body

    @property
    def http(self) -> ClientResponse:
        """
        Property returns aiohttp.ClientResponse
        """
        return self._response

    @property
    def status(self) -> int:
        return self.http.status

    @property
    def body(self) -> FreshChatResponseBody:
        """
        Property returns FreshChatResponseBody which can be either a String or a
        Dictionary
        """
        return self._body

    @staticmethod
    def _decode(body: str):
        try:
            return json.loads(body)
        except json.JSONDecodeError:
            return body

    @classmethod
    async def load(cls, response: ClientResponse) -> "FreshChatResponse":
        """
        Class method creates and returns an instance of the class given
        an aiohttp.ClientResponse
        """
        if response.content_type != "application/json":
            return cls(response, await response.text())
        return cls(response, await response.json(loads=cls._decode))

    def __getattr__(self, attr):
        return getattr(self.http, attr)

    def __repr__(self):
        return f"{self.__class__.__name__}<{hex(id(self))}>(body={self.body})"
